#!/usr/bin/env python
# -*- coding:utf-8 -*-

import configparser
import os, json, base64
from decimal import Decimal
import datetime, time
from urllib import parse

import pymongo
import mongoengine

import pymysql
import logging
import datetime, time, uuid, re
import hashlib
from common.error import *
from conf.conf import BaseConf
from tornado.options import define, options
import zipfile


def my_options(arg):
    return dict(options.items())[arg]


class Config(object):
    """
    # Config().get_content("user_information")
    """

    def __init__(self):
        file_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "conf", BaseConf.BaseCnf)
        # file_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "conf", my_options("conf"))
        self.cf = configparser.ConfigParser()
        self.cf.read(file_path,encoding='utf-8')

    def get_sections(self):
        return self.cf.sections()

    def get_options(self, section):
        return self.cf.options(section)

    def get_content(self, section):
        result = {}
        for option in self.get_options(section):
            value = self.cf.get(section, option)
            result[option] = int(value) if value.isdigit() else value
        return result


class MongoDb(object):
    def __init__(self, uri, host, port, db_name, user, password):
        self._uri = uri
        self._db_host = host
        self._db_port = int(port)
        self._user = user
        self._password = password
        self._db = db_name
        self.conn = None

    def connect(self):
        if self._uri:
            self.conn = pymongo.MongoClient(self._uri)
        else:
            if self._user and self._password:
                self._password=parse.quote(self._password)
                x='mongodb://'+self._user+':'+self._password+'@'+self._db_host+':'+str(self._db_port)+'/'
                self.conn = pymongo.MongoClient(x)  #self._db_host, self._db_port,self._db, self._user, self._password
            else:
                self.conn=pymongo.MongoClient(self._db_host, self._db_port)
        return self.conn

    def get_db(self, db_name):
        collection = self.conn.get_database(db_name)
        if self._user and self._password:
            collection.authenticate(self._user, self._password)
        return collection

    def close(self):
        if self.conn:
            self.conn.close()
            self.conn = None


class MongoConfig(object):
    """
    # Config().get_content("user_information")
    """

    def __init__(self):
        file_path = os.path.join(os.path.dirname(os.path.dirname(__file__)), "conf", BaseConf.BaseMongoCnf)
        self.cf = configparser.ConfigParser()
        self.cf.read(file_path,encoding='utf-8')

    def get_sections(self):
        return self.cf.sections()

    def get_options(self, section):
        return self.cf.options(section)

    def get_content(self, section):
        result = {}
        for option in self.get_options(section):
            value = self.cf.get(section, option)
            result[option] = int(value) if value.isdigit() else value
        return result


class Pymongoengine(MongoDb):
    def __init__(self, conf_name):
        # self.conf = MongoConfig().get_content(conf_name)
        # {'host': '192.168.2.180', 'port': '32769', 'dbname': 'edata', 'user': '', 'password': ''}
        self.conf=conf_name
        super(Pymongoengine, self).__init__(**self.conf)

    def connect_db(self):
        if self._uri:
            self.conn = mongoengine.connect(host=self._uri)
        else:
            if self._user and self._password:
                self.conn = mongoengine.connect(self._db, host=self._db_host, port=self._db_port,username=self._user,
                                                password=self._password, authentication_source=self._db)
            else:
                self.conn = mongoengine.connect(self._db, host=self._db_host, port=self._db_port)
        return mongoengine.DynamicDocument

    def disconnect(self):
        mongoengine.disconnect()

    def get_db(self, db_name):
        collection = self.conn.get_database(db_name)
        if self._user and self._password:
            collection.authenticate(self._user, self._password)
        return collection


class base_pymysql(object):
    def __init__(self, host, port, user, password, db_name=None, cursor=pymysql.cursors.DictCursor):
        self.db_host = host
        self.db_port = int(port)
        self.user = user
        self.password = str(password)
        self.db = db_name
        self.conn = None
        self.cursor = cursor

    def connect(self):
        self.conn = pymysql.connect(host=self.db_host, port=self.db_port, user=self.user,
                                    passwd=self.password, db=self.db, charset="utf8")
        self.cursor = self.conn.cursor(cursor=self.cursor)


class MyPymysql(base_pymysql):
    """
    Basic Usage:

        ret = My_Pymysql('test1')
        res = ret.selectone_sql("select * from aaa")
        print(res)
        ret.close()
    Precautions:
        Config.__init__(self, config_filename="analy.cnf")

    """
    '''
    explain:

        def mysubData(subData):
            b = []
        
            for i in subData:
                try:
                    if numpy.isnan(i):
                        i = "Null"
                        b.append(i)
                    elif isinstance(i, numpy.float):
                        b.append(i)
                    elif isinstance(i, numpy.int):
                        b.append(i)
                except:
                    if isinstance(i, str):
                        if "'" in i:
                            i.replace("'", "")
                        elif "%" in i:
                            i.replace("'", "")
                        elif "\\" in i:
                            i.replace("'", "")
                        b.append(i)
        
            return b
        
        
        class writer_information_tables():
            def __init__(self, libname="notdbMysql"):
                self.libname = libname
                self.res = MyPymysql(self.libname)
        
            def MetaDataTableName_sql(self):
                """
                1. 先判断meta_data库里面最后一条数据的databaseName的库名， 得到count， 如果大于10000， 那么在创建新库
        
                """
                selectMetaDataTableName_sql = """select DatabaseName from db_metadata.meta_data_table Order by DatabaseName limit 1;"""
                MetaDataTableName = self.res.selectone_sql(selectMetaDataTableName_sql)
                MetaDataTableName = MetaDataTableName if MetaDataTableName else {"DatabaseName": "db_data_20170524164100"}
                MetaDataTableNameCount_sql = """SELECT COUNT(1) as count FROM information_schema.tables WHERE TABLE_SCHEMA = '%s';""" % \
                                             MetaDataTableName["DatabaseName"]
                MetaDataTableNameCount = self.res.selectone_sql(MetaDataTableNameCount_sql)
                return MetaDataTableNameCount["count"], MetaDataTableName["DatabaseName"]
        
            def CreateNewDataBase_sql(self):
                """
                创建新的库
                :return:
                """
                dataBaseNewName = "db_data" + str(datetime.datetime.now().strftime("%Y%m%d%H%M%S"))
                createNewDataBase_sql = """CREATE DATABASE %s CHARACTER SET=UTF8;""" % dataBaseNewName
                self.res.insert_sql(createNewDataBase_sql)
                return dataBaseNewName
        
            def CreateNewDataTable_sql(self, tableName):
                """
                创建新的数据表
                先判断这个库是否有10000个表
                    如果有，新建库
                    没有，继续使用
                :return:
                """
                # 判断库名函数， 如果库里表>10000, 则新建
                count, dataBaseName = self.MetaDataTableName_sql()
                dataBaseName = dataBaseName if count < 10000 else self.CreateNewDataBase_sql()
        
                CreateTable_sql = """CREATE TABLE {}.{} (""".format(dataBaseName, tableName)
                CreateTable_sql += """
                DataID int(5) PRIMARY KEY AUTO_INCREMENT UNIQUE KEY,
                `ID` int(5) DEFAULT NULL,
                `StartTime` varchar(50) DEFAULT NULL,
                `EndTime` varchar(50) DEFAULT NULL,
                index ix_NominalTime (NominalTime),
                index ix_Index (`Index`)
                """
        
                CreateTable_sql += ") ENGINE=InnoDB DEFAULT CHARSET=UTF8"
                self.res.insert_sql(CreateTable_sql)
                return dataBaseName
        
            def InsertNewStataData_sql(self, dataBaseName, tableName, subData):
                subData = tuple(mysubData(subData))
        
                variable = ("`ID`", "`StartTime`", "`EndTime`")
        
                sql = """insert INTO {}.{} {} VALUES""".format(dataBaseName, tableName, variable)
                sql = sql.replace("'", "")
                sql = sql + """{};""".format(subData)
                sql = sql.replace("None", "Null")
        
                self.res.idu_sql(sql)
        
            def close(self):
                self.res.close()
    '''

    def __init__(self, conf_name, cursor=pymysql.cursors.DictCursor):
        self.conf = Config().get_content(conf_name)
        super(MyPymysql, self).__init__(**self.conf, cursor=cursor)
        self.connect()

    def idu_sql(self, sql, value=None):
        # adu: insert, delete, update的简写
        # 考虑到多语句循环, try就不写在这里了
        self.cursor.execute(sql, value)
        self.conn.commit()

    def insert_sql(self, sql, value=None):
        # adu: insert, delete, update的简写
        # 防止sql注入
        self.cursor.execute(sql, value)
        self.conn.commit()

    def selectone_sql(self, sql):
        self.cursor.execute(sql)
        self.conn.commit()
        return self.cursor.fetchone()

    def selectall_sql(self, sql):
        self.cursor.execute(sql)
        self.conn.commit()
        return self.cursor.fetchall()

    def select_sql(self, sql, value=None):
        # 防止sql注入
        self.cursor.execute(sql, value)
        self.conn.commit()
        return self.cursor.fetchall()

    def close(self):
        self.conn.close()
        self.cursor.close()
        self.conn = None
        self.cursor = None


class MyPymysqlTwo(base_pymysql):
    """
    The class add affair

    Basic Usage:

        ret = My_Pymysql('test1')
        res = ret.selectone_sql("select * from aaa")
        print(res)
        ret.close()
        --------------
        class writer_information_tables():
            def __init__(self, libname="metadata"):
                self.libname = libname
                self.pymysql = MyPymysqlTwo('metadata')

            def __enter__(self):
                return self

            def insert_sql(self, data):
                sql = '''insert INTO `meta_variable` SET DataTableID={}, VarValues="%s";'''.format(
                    data["DataTableID"])
                value = (data["VarValues"])
                self.pymysql.insert_sql(sql, value=value)

            def close(self):
                self.pymysql.close()

            def __exit__(self, exc_type, exc_val, exc_tb):
                self.close()

    Precautions:
        Config.__init__(self, config_filename="my.cnf")

    """

    def __init__(self, conf_name):
        self.conf = Config().get_content(conf_name)
        super(MyPymysqlTwo, self).__init__(**self.conf)
        self.connect()

    def idu_sql(self, sql, value=None):
        # adu: insert, delete, update的简写
        # 考虑到多语句循环, try就不写在这里了
        self.cursor.execute(sql, value)
        self.conn.commit()

    def commit(self):
        self.conn.commit()

    def rollback(self):
        self.conn.rollback()

    def insert_sql(self, sql, value=None):
        # adu: insert, delete, update的简写
        # 防止sql注入
        self.cursor.execute(sql, value)

    def update_sql(self, sql, value=None):
        # adu: insert, delete, update的简写
        # 防止sql注入
        self.cursor.execute(sql, value)

    def delete_sql(self, sql, value=None):
        # adu: insert, delete, update的简写
        # 防止sql注入
        self.cursor.execute(sql, value)

    def selectone_sql(self, sql):
        self.cursor.execute(sql)
        self.conn.commit()
        return self.cursor.fetchone()

    def selectall_sql(self, sql):
        self.cursor.execute(sql)
        self.conn.commit()
        return self.cursor.fetchall()

    def select_sql(self, sql, value=None):
        # 防止sql注入
        self.cursor.execute(sql, value)
        self.conn.commit()
        return self.cursor.fetchall()

    def close(self):
        self.conn.close()
        self.cursor.close()
        self.conn = None
        self.cursor = None


def result(status, value=None, message=None):
    """
    staatus:
    2000, 什么都ok
    4000, 客户上传的文件格式不正确
    4001， 客户上传的文件列超过5400
    4002， 值传递错误
    5000， 服务器错误
    5001， 数据表已经存在
    5002,  sql语句错误
    """
    if not message:
        try:
            message = CODEMSG[int(status)]
        except:
            message = "Unknown Error!!!"
    return {
        "statuscode": status,
        "statusmessage": message,
        "value": value
    }


class my_datetime():
    """
    Basic usage:

        a = datetime.datetime(2016, 9, 21, 13, 42, 8)
        b = "2016-11-15 15:32:12"
        c = u'2016-09-21 13:37:34'
        print type(c)
        d = 1474436826.0
        e = 13710788676.0
        ret = my_datetime()
        res = ret.become_datetime(e)
        print res
        print type(res)
    """

    def __init__(self):
        # 缺少对utc时间的判断
        pass

    def become_timestamp(self, dtdt):
        # 将时间类型转换成时间戳
        if isinstance(dtdt, datetime.datetime):
            timestamp = time.mktime(dtdt.timetuple())
            return timestamp

        elif isinstance(dtdt, str):
            if dtdt.split(" ")[1:]:
                a_datetime = datetime.datetime.strptime(dtdt, "%Y-%m-%d  %H:%M:%S")
                timestamp = time.mktime(a_datetime.timetuple())
            else:
                a_datetime = datetime.datetime.strptime(dtdt, "%Y-%m-%d")
                timestamp = time.mktime(a_datetime.timetuple())
            return timestamp

        elif isinstance(dtdt, float):
            return dtdt

            # elif isinstance(dtdt, unicode):
            #     if dtdt.split(" ")[1:]:
            #         a_datetime = datetime.datetime.strptime(dtdt, "%Y-%m-%d %H:%M:%S")
            #         timestamp = time.mktime(a_datetime.timetuple())
            #     else:
            #         a_datetime = datetime.datetime.strptime(dtdt, "%Y-%m-%d")
            #         timestamp = time.mktime(a_datetime.timetuple())
            #     return timestamp

    def become_datetime(self, dtdt):
        # 将时间类型转换成datetime类型
        if isinstance(dtdt, datetime.datetime):
            return dtdt

        elif isinstance(dtdt, str):
            if dtdt.split(" ")[1:]:
                a_datetime = datetime.datetime.strptime(dtdt, "%Y-%m-%d %H:%M:%S")
            else:
                a_datetime = datetime.datetime.strptime(dtdt, "%Y-%m-%d")
            return a_datetime

        elif isinstance(dtdt, float):
            # 把时间戳转换成datetime类型
            a_datetime = datetime.datetime.fromtimestamp(dtdt)
            return a_datetime

            # elif isinstance(dtdt, unicode):
            #     if dtdt.split(" ")[1:]:
            #         a_datetime = datetime.datetime.strptime(dtdt, "%Y-%m-%d %H:%M:%S")
            #     else:
            #         a_datetime = datetime.datetime.strptime(dtdt, "%Y-%m-%d")
            #     return a_datetime

    def become_str(self, dtdt):
        # 把时间类型转换成字符串
        if isinstance(dtdt, datetime.datetime):
            a_datetime = dtdt.strftime("%Y-%m-%d %H:%M:%S")
            return a_datetime

        elif isinstance(dtdt, str):
            return dtdt

        elif isinstance(dtdt, float):
            a_datetime_local = datetime.datetime.fromtimestamp(dtdt)
            a_datetime = a_datetime_local.strftime("%Y-%m-%d %H:%M:%S")
            return a_datetime

            # elif isinstance(dtdt, unicode):
            #     # 区别：一个是strp， 一个是strf
            #     if dtdt.split(" ")[1:]:
            #         a_datetime = datetime.datetime.strptime(dtdt, "%Y-%m-%d %H:%M:%S")
            #         a_datetime = a_datetime.strftime("%Y-%m-%d %H:%M:%S")
            #     else:
            #         a_datetime = datetime.datetime.strptime(dtdt, "%Y-%m-%d")
            #         a_datetime = a_datetime.strftime("%Y-%m-%d")
            #     return a_datetime

    @staticmethod
    def str_datetime():
        return (datetime.datetime.now()).strftime("%Y-%m-%d %H:%M:%S")


def MyGuid():
    date = datetime.datetime.now().strftime('%Y%m%d%H%M%S')

    us = ("%.7f" % time.time())[-7:]

    res = "".join(re.findall('\d+', str(uuid.uuid4())))[:7]

    ret = date + us + res

    return ret


def FormatMyStr(arg, type=None):
    if isinstance(arg, str):
        return arg.replace(' ', '').replace('\\', '').replace('/', '').replace("'", '').replace('"', '')
    elif isinstance(arg, list):
        if type:
            if type == int:
                return str([int(i) for i in arg]).replace("[", "(").replace("]", ")")
            elif type == float:
                return str([float(i) for i in arg]).replace("[", "(").replace("]", ")")
            elif type == str:
                return str([str(i) for i in arg]).replace("[", "(").replace("]", ")")
            else:
                return str(arg).replace("[", "(").replace("]", ")").replace("，", ",")
        else:
            return arg
    else:
        return ""


def GetFileSize(filePath, size=0):
    for root, dirs, files in os.walk(filePath):
        for f in files:
            size += os.path.getsize(os.path.join(root, f))
    return size


"""
    获取文件Md5值
    file_path = r'/opt/code/my_code/zk_css/analy.cnf'
    md5_02 = GetBigFileMd5Value(file_path)

"""


class ComplexEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, (datetime.datetime, datetime.date)):
            return str(obj.strftime("%Y-%m-%d %H:%M:%S"))
        elif isinstance(obj, Decimal):
            return str(obj)
        else:
            # return json.JSONEncoder.default(self, obj)
            return str(obj)


def GetBigFileMd5Value(file_path):
    f = open(file_path, 'rb')
    md5_obj = hashlib.md5()
    while True:
        d = f.read(8096)
        if not d:
            break
        md5_obj.update(d)
    hash_code = md5_obj.hexdigest()
    f.close()
    md5 = str(hash_code).lower()
    return md5


def JoinWhooshAPi(WhooshApiRet):
    i = WhooshApiRet["protocol"] + "://"
    # i = "http://"
    i += WhooshApiRet["host"] + ":" + str(WhooshApiRet['port'])
    return i


# -----<base64加密解密>----- #

class MyBase64:
    # 加密
    @staticmethod
    def encryption(arg):
        bytesString = arg.encode(encoding="utf-8")

        # base64 编码
        encodestr = base64.b64encode(bytesString)
        return encodestr.decode()

    @staticmethod
    def decryption(arg):
        decodestr = base64.b64decode(arg)

        return decodestr.decode()

# class Base:
#     metadata = Config().get_content("metadata")["db_name"]
#     metadataRead = Config().get_content("metadataRead")["db_name"]
#     data = Config().get_content("data")["db_name"]

def rsa_verify(pubkey, ProjectID):
    import rsa
    privkey = '-----BEGIN RSA PRIVATE KEY-----\nMIICYQIBAAKBgQC6LwTNnNVOfRe1g/caZrS0C0txLcIAQjERR9YO6U4wqJoicYqS\n0go5WGKEUTwsSQQHM89AGV/a4g1/Ei61Nq8pXq1Y0ZJ7ajg0NTyyERXJibQodEN3\nXaj96kKjO9+ZA8sd/xvWyIjwXrBv1E9Rf2Rp4LMu32utDkcXmgDXUvUpeQIDAQAB\nAoGAUgxAcKAJvHISDbLfJWpXffRwRaslkFmbKANhSOb1xym16zHgIH2hdc4GdEsc\nWmO++7GQRq4n+Nyfl/0Z48Z2lN0w0VQj1pGZ+R4BA4QgLFZaVN+kyqdA7JtkShAs\n87ca4PCbC8503B4mnfrKsGoMi3uSJlI8Tiktm6ILZofAf4ECRQDorLRltXVHHVQb\nUXhibt+uiZ1vle9KfBl7pPmeHtnlsfn5xXKbJxI1h39WxvcsBaE39OR33rBNqrru\n8Fr6lVLcPPTa8QI9AMzZLu5UxELsK+GvwnGq/jQM1BZZGXdRS/bfTYiG2t/MFtAD\nvQ1bKQZEitrOGVOj4/FrYFwX6myc5WrnCQJFAMovUS9UqE2cobp/aCH5AKyVbtZJ\nLr/F6Qr1h+esHWKo7nTp67YBFKSpUrEggiah2b9bMbEsNH25D2mUOSqo2XECEy+x\nAj0Amivq8lzByUxOSnEpersizrHI/XjZfb+l2It/ov07xauoj64Anwi4+4qipsLu\ntNBoQc56m1U0jcIXuDlhAkRxPKjrw34XYbxsw/2p1t4/byvdS7a50Lk391SYVqGr\nXjkSlbtmedlE9wsG0aPnp5gJdLfXUxQwiocSbO2KYTGXgNIM4g==\n-----END RSA PRIVATE KEY-----\n'

    # pubkey = rsa.PublicKey.load_pkcs1(pubkey.encode())
    privkey = rsa.PrivateKey.load_pkcs1(privkey.encode())

    # 明文
    # message = ProjectID

    message = rsa.decrypt(pubkey, privkey).decode()
    if message == ProjectID:
        return 1
    else:
        return 0

# database = Base.metadata


def format_content(arg):
    if type(arg) == str:
        return str(arg).replace('<', "&lt").replace('>', "&gt")
    elif type(arg) == list or type(arg) == tuple:
        li = []
        for i in arg:
            if type(i) == str:
                li.append(str(i).replace('<', "&lt").replace('>', "&gt"))
            else:
                li.append(i)
        return li
    else:
        return arg


def addzip(li, filepathname):
    f = zipfile.ZipFile(filepathname,'w',zipfile.ZIP_DEFLATED)
    for i in li:
        file = i.split('/')[-1]
        f.write(i, file)
    f.close()


def JoinTableName(myBaseList):
    myTable = ""
    for i in range(len(myBaseList)):
        if myTable:
            myTable += str(myBaseList[i]["DatabaseName"]) + "." + str(myBaseList[i]["DataTableName"]) + " AS a" + str(i) + " ON " + "a" + str(
                i - 1) + ".mycustomid=a" + str(i) + ".mycustomid  INNER JOIN "
        else:
            myTable += str(myBaseList[i]["DataTableName"]) + " AS a" + str(i) + " INNER JOIN "

    myTable = myTable.rstrip("INNER JOIN ")
    return myTable

def nanToStr(nan):
    if nan != nan:
        return 'NaN'
    return nan

try:
    logDict = Config().get_content("log")
    logPath = logDict['logpath']
    logName = logDict['logname']

except Exception as e:
    logPath = None
    logName = None

# my_log = Logger(filepath=logPath, filename=logName)

max_columns_count = 500



if __name__ == '__main__':
    # ret = FormatMyStr('"asdf/n')
    # print(ret)
    print(my_datetime.str_datetime())
